/*
 * Project name:
     PID BAR Example 1
 * Author:
     Stevan Tosic
 * Date:
     11.05.2012.
 * Revision History:
       - initial release;
 * Description:
     This is a simple process PID simulator application with
     three PID controllers
 * Test configuration:
     MCU:             P18F8520
     dev.board:       bigpic5
     Oscillator:      HS, 8.0 MHz
     Ext. Modules:    -
     SW:              mikroC PRO for PIC 5.0.1.
 * NOTES:
     - PID simulation on GLCD
*/

#include "pid.h"
#include "bar.h"

#define NP 3

// Glcd module connections
char GLCD_DataPort at PORTD;
sbit GLCD_CS1 at LATJ0_bit;  // for writing to output pin always use latch (PIC18 family)
sbit GLCD_CS2 at LATJ1_bit;  // for writing to output pin always use latch (PIC18 family)
sbit GLCD_RS  at LATJ2_bit;  // for writing to output pin always use latch (PIC18 family)
sbit GLCD_RW  at LATJ3_bit;  // for writing to output pin always use latch (PIC18 family)
sbit GLCD_EN  at LATJ4_bit;  // for writing to output pin always use latch (PIC18 family)
sbit GLCD_RST at LATJ5_bit;  // for writing to output pin always use latch (PIC18 family)

sbit GLCD_CS1_Direction at TRISJ0_bit;
sbit GLCD_CS2_Direction at TRISJ1_bit;
sbit GLCD_RS_Direction  at TRISJ2_bit;
sbit GLCD_RW_Direction  at TRISJ3_bit;
sbit GLCD_EN_Direction  at TRISJ4_bit;
sbit GLCD_RST_Direction at TRISJ5_bit;
// End Glcd module connections

PID pids[NP];
PROCESS processes[NP];
BAR xbars[NP], ebars[NP], ubars[NP], ybars[NP];
//BAR bars[NP][4];
int i, k, ref, M=100;
int midy, d1;
char txts[NP][5]={"PID1", "PID2","PID3"}; //,"PID4"}; //,"5","6","7","8"};
char text[6];

void delay2S(){                             // 2 seconds delay function
  Delay_ms(2000);
}

void bars_plot(void)
{
   int n;
   bar_plot(&xbars[i], ref);
   n=processes[i].N;
   bar_plot(&ubars[i], processes[i].u[n]/5);
   IntToStr(processes[i].u[n], text);
   Glcd_Write_Text(text, 40*i, 7, 0);
   bar_plot(&ybars[i], processes[i].y[n+1]);
   bar_plot(&ebars[i], processes[i].e);
}

void bars_processes_init(void)
{
   Glcd_Write_Text("x y e: 1,2,3", 0, 0, 2);      // Write string
   Glcd_Write_Text("    u1", 0, 6, 1);
   Glcd_Write_Text("    u2", 40, 6, 1);
   Glcd_Write_Text("    u3", 80, 6, 1);

   d1=0;
   for(i=0; i<NP; i++)
   {
      bar_init(&xbars[i], 0, 0, 8*(4*i+1)+d1, midy, WHITE, 3);
      bar_init(&ybars[i], 0, 0, 8*(4*i+2)+d1, midy, WHITE, 3);
      bar_init(&ebars[i], 0, 0, 8*(4*i+3)+d1, midy, WHITE, 3);
      bar_init(&ubars[i], 0, 0, 8*(4*i+4)+d1, midy, WHITE, 3);
      d1+=8;
      //Glcd_Write_Text(txts[i], 10*(3*i), 7, 2);      // Write string
      process_set(&processes[i], 1, 2, 10, 0.7);
      pid_set(&pids[i], &processes[i], 3,2,3);
   }
}

void main() {
  ADCON1 |= 0x0F;                           // Configure AN pins as digital
  CMCON  |= 7;                              // Disable comparators

  Glcd_Init();                              // Initialize GLCD
  midy = 64 / 2;

  while(1) {
   Glcd_Fill(0x00); // Clear GLCD
   bars_processes_init();

   for(k=0; k<M; k++)
   {
      if(k<50) ref=18;
      else ref=8;

      for(i=0; i<NP;i++)
      {
         error(&processes[i], ref);
         pid(&processes[i], &pids[i]);
         process(&processes[i]);
         bars_plot();
         process_shift(&processes[i]);
      }
      Delay_ms(270);
   }
    Delay_ms(3000);
  }
}