/*
 * Project name:
     PID BAR Example 1
 * Author:
     Stevan Tosic
 * Date:
     11.05.2012.
 * Revision History:
       - initial release;
 * Description:
     This is a simple PID and process simulation library
 * Test configuration:
     MCU:             P18F8520
     dev.board:       bigpic5
     Oscillator:      HS, 8.0 MHz
     Ext. Modules:    -
     SW:              mikroC PRO for PIC 5.0.1.
 * NOTES:
     - PID and process simulation functions
*/

#include "pid.h"

static uint16_t Kp, Ki, Kd, i;
static int32_t du, yk, yk_1, yk_2;

void process_set(PROCESS* p, float T, int16_t N1, float T1, float K)
{
  p->e=0; p->eold=0;
  p->T=T; 
  p->N=N1;
  p->T1=T1;
  p->K=K;
  p->a=exp(-(p->T)/(p->T1));
  p->b=(p->K)*(1-(p->a));
  for(i=0; i<=p->N; i++)
  {
     p->y[i]=p->u[i]=0;
  }
}

void pid_set(PID* pid, PROCESS* p, float Kp, float Ti, float Td)
{
   float T;
   T=p->T;
   //pid->e=0;
   pid->Kp=1024*Kp;
   pid->Ki=1024*Kp*T/Ti;
   pid->Kd=1024*Kp*Td/T;
}

int16_t error(PROCESS* p, int16_t x)
{
   int N;
   N=p->N;
   p->e=x-(p->y[N]);
   return p->e;
}

int16_t pid(PROCESS* p, PID* pid)
{
   float T, e;
   int N;
   
   Kp= pid->Kp;
   Ki= pid->Ki;
   Kd= pid->Kd;  // !!
   
   T=p->T;
   N=p->N;
   e=p->e;
   yk=p->y[N]; yk_1=p->y[N-1]; yk_2=p->y[N-2];
   //du=-(pidreg->Kp)*(yk-yk_1-T*e/(pidreg->Ti)+(pidreg->Td)*(yk-2*yk_1+yk_2)/T);
   du=Kp*(yk_1-yk)+Ki*e-Kd*(yk-2*yk_1+yk_2);
   du>>=10;
   p->u[N]=(p->u[N-1])+du;
   return p->u[N];
}

int16_t process(PROCESS* p)
{
   float a, b;
   int N;
   a=p->a;
   b=p->b;
   N=p->N;
   p->y[N+1]=a*(p->y[N])+b*(p->u[0]);
   return p->y[N+1];
}

void process_shift(PROCESS* p)
{
    int N;
    N=p->N;
    for(i=0; i<N; i++)
    {
        p->y[i]=p->y[i+1];
        p->u[i]=p->u[i+1];
    }
    p->y[i]=p->y[i+1];
    p->eold=p->e;
}